import c4d
from c4d import gui
import math
#Welcome to the world of Python

def GetDistance(vec1, vec2):
    dist = vec1 - vec2
    return math.sqrt(dist.x**2 + dist.y**2 + dist.z**2)

def FromMgToMl(mother, child_global_pos, child_global_rot):
    mg = mother.GetMg()
    rot = child_global_rot.GetMg()
    pos = child_global_pos.GetMg().off
    return c4d.Matrix.__mul__(mg.__invert__(),c4d.Matrix(pos, rot.v1, rot.v2, rot.v3))

def Pos2Off(obj, local_pos):
    if isinstance(obj, c4d.BaseObject): mg = obj.GetMg()
    if isinstance(obj, c4d.Matrix): mg = obj
    if isinstance(local_pos, c4d.Vector): pos = local_pos
    if isinstance(local_pos, c4d.Matrix): pos = local_pos.off
    if isinstance(local_pos, c4d.BaseObject): pos = local_pos.GetMg().off
    pos = c4d.Vector(int(pos.x),int(pos.y),int(pos.z))
    return mg.Mul(pos)

def Frz2Rel(obj):
    obj.SetRelPos(obj.GetFrozenPos())
    obj.SetRelRot(obj.GetFrozenRot())
    obj.SetFrozenPos(c4d.Vector(0,0,0))
    obj.SetFrozenRot(c4d.Vector(0,0,0))

def Rel2Frozen(obj):
    obj.SetFrozenPos(obj.GetRelPos())
    obj.SetFrozenRot(obj.GetRelRot())
    obj.SetRelPos(c4d.Vector(0,0,0))
    obj.SetRelRot(c4d.Vector(0,0,0))

def Abs2Frozen(obj):
    obj.SetFrozenPos(obj.GetAbsPos())
    obj.SetFrozenRot(obj.GetAbsRot())
    obj.SetRelPos(c4d.Vector(0,0,0))
    obj.SetRelRot(c4d.Vector(0,0,0))

def GetGlPointsTuple(obj):
    points = obj.GetAllPoints()
    gl_points = ()
    for i in points: gl_points = gl_points + (Pos2Off(obj, i),)
    return gl_points

TGROUP = 1000
TEXT = 1001
TEXT2= 1011
BGROUP = 1002
NEIGHBOUR = 1003
SELECTION = 1004
CANCEL = 1005
LGROUP = 1006
FLOAT = 1007
LTEXT = 1008

class Dlg(gui.GeDialog):

    real = 3.0

    def CreateLayout(self):
        self.GroupBegin(TGROUP, c4d.BFV_SCALE, cols=1, rows=2)
        self.AddStaticText(TEXT, c4d.BFH_CENTER, name="Soll nach Nachbarn oder gem\\ der Reihenfolge")
        self.AddStaticText(TEXT2, c4d.BFH_CENTER, name="der selektierten Objekte geordnet werden?")
        self.GroupEnd()
        self.GroupBegin(BGROUP, c4d.BFV_BOTTOM, cols=4)
        self.AddButton(NEIGHBOUR, c4d.BFH_SCALEFIT, name="Nachbar")
        
        self.AddButton(SELECTION, c4d.BFH_SCALEFIT, name="Selektion")
        self.AddButton(CANCEL, c4d.BFH_SCALEFIT, name="Abbruch")
        self.GroupEnd()
        self.GroupBegin(LGROUP, c4d.BFV_BOTTOM, cols=2)
        self.AddEditNumber(FLOAT, c4d.BFH_SCALEFIT, initw=40)
        self.AddStaticText(LTEXT, c4d.BFH_SCALEFIT, name="Punktdistanz-Toleranz bei Nachbarsuche.")
        self.GroupEnd()
        return True

    def InitValues(self):
        self.SetMeter(FLOAT, self.real)
        return True 

    def Command(self, id, msg):
        if id==CANCEL:
            self.Close()

        if id==FLOAT:
            self.real = self.GetReal(FLOAT)

        if id==NEIGHBOUR:
            Neighbour(self.real)
            self.Close() 

        if id==SELECTION:
            Selection()
            self.Close() 
            
        return True 

def Selection():
    c4d.StopAllThreads() 
    sel_objs = doc.GetOrderedActiveObjects()

    clone_tup=()
    for i in range(len(sel_objs)): clone_tup=clone_tup+(sel_objs[i].GetClone(),)

    doc.InsertObject(clone_tup[0], sel_objs[0].GetUp(), None)

    for i in range(1, len(clone_tup)):
        doc.InsertObject(clone_tup[i], clone_tup[i-1], None)
        clone_tup[i].SetMl(FromMgToMl(sel_objs[i-1], sel_objs[i], sel_objs[i]))
        Abs2Frozen(clone_tup[i])

    for i in range(len(sel_objs)):
        sel_objs[i].Remove()

def Neighbour(real):
    c4d.StopAllThreads()
    sel = doc.GetOrderedActiveObjects() 
    first = sel[0]
    new_sel = (first,)
    while len(sel) > 1:
        sel.remove(first)
        next = None
        
        mpnts = GetGlPointsTuple(first)
        for obj in sel:
            count = 0
            cpnts = GetGlPointsTuple(obj)
            for cpoint in cpnts:
                for mpoint in mpnts:
                    if GetDistance(cpoint, mpoint) < real: count = count + 1
                if count == 2: next = obj
        if next == None:
            gui.MessageDialog("Entweder die Kette ist nicht zusammenh\ngend, das erste ausgew\hlte Polygon ist nicht Anfangs- oder Endglied oder es handelt sich um keine Einreiherkette.") 
            return False
        else:
            new_sel = new_sel + (next,)
            first = next
        
    clone_tup=()
    for i in range(len(new_sel)): clone_tup=clone_tup+(new_sel[i].GetClone(),)

    doc.InsertObject(clone_tup[0], new_sel[0].GetUp(), None)

    for i in range(1, len(clone_tup)):
        Frz2Rel(clone_tup[i])
        doc.InsertObject(clone_tup[i], clone_tup[i-1], None)
        clone_tup[i].SetMl(FromMgToMl(new_sel[i-1], new_sel[i], new_sel[i]))
        Rel2Frozen(clone_tup[i])
            
    for i in range(len(new_sel)):
        new_sel[i].Remove()    

def main():
    sel = doc.GetOrderedActiveObjects() 

    if len(sel) < 2:
        gui.MessageDialog("W\hle mindestens zwei Polygon-Objekte aus.")
        return False
    mother = sel[0].GetUp()
    for obj in sel:
        if obj.GetUp() != mother: 
            gui.MessageDialog("Die Auswahl darf nur Polygon-Objekte der selber Hierachie umfassen.")
            return False
        if obj.GetType() != 5100:
            gui.MessageDialog("Die Auswahl darf nur Polygon-Objekte umschlie\en.")
            return False

    dlg = Dlg()
    dlg.Open(c4d.DLG_TYPE_MODAL, xpos=850, ypos= 150, defaultw=200, defaulth=60)
    c4d.EventAdd()


if __name__=='__main__':
    main()